#!/usr/bin/env python3

"""
This Python script is designed to query multiple online repositories for the
synonyms associated with the given word.
"""
__author__ = 'John Bumgarner'
__date__ = 'October 15, 2020'
__status__ = 'Production'
__license__ = 'MIT'
__copyright__ = "Copyright (C) 2020 John Bumgarner"

##################################################################################
# “AS-IS” Clause
#
# Except as represented in this agreement, all work produced by Developer is
# provided “AS IS”. Other than as provided in this agreement, Developer makes no
# other warranties, express or implied, and hereby disclaims all implied warranties,
# including any warranty of merchantability and warranty of fitness for a particular
# purpose.
##################################################################################

##################################################################################
# Python imports required for basic operations
##################################################################################
import bs4
import logging
import requests
import traceback
import re as regex
from bs4 import BeautifulSoup
from wordhoard.utilities import basic_soup, caching, cleansing, wordhoard_logger, word_verification

logger = logging.getLogger(__name__)
wordhoard_logger.enable_logging(logger)


def query_collins_dictionary_synonym(single_word):
    """
    This function queries collinsdictionary.com for synonyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of synonyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_synonyms(single_word, 'collins_dictionary')
        if not check_cache:
            try:
                synonyms = []
                results_synonym = basic_soup.get_single_page_html(
                    f'https://www.collinsdictionary.com/dictionary/english-thesaurus/{single_word}')
                soup = BeautifulSoup(results_synonym, 'lxml')
                word_found = soup.find('h1', text=f'Sorry, no results for “{single_word}” in the English Thesaurus.')
                if word_found:
                    logger.error(f'Collins Dictionary had no reference for the word {single_word}')
                    logger.error(f'Please verify that the word {single_word} is spelled correctly.')
                else:
                    query_results = basic_soup.query_html(results_synonym, 'div', 'class', 'blockSyn')
                    content_descendants = query_results.descendants
                    for item in content_descendants:
                        if item.name == 'div' and item.get('class', 'form type-syn orth'):
                            children = item.findChild('span', {'class': 'orth'})
                            if children is not None:
                                synonyms.append(children.text)
                    caching.insert_word_cache_synonyms(single_word, 'collins_dictionary', synonyms)
                    return sorted(synonyms)
            except bs4.FeatureNotFound as error:
                logger.error('A BeautifulSoup error occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except AttributeError as error:
                logger.error('An AttributeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except KeyError as error:
                logger.error('A KeyError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except TypeError as error:
                logger.error('A TypeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            synonyms = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return synonyms
    else:
        logger.error(f'The word "{single_word}" was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')


def query_synonym_com(single_word):
    """
    This function queries synonym.com for synonyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of synonyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_synonyms(single_word, 'synonym_com')
        if not check_cache:
            try:
                results_synonym = basic_soup.get_single_page_html(f'https://www.synonym.com/synonyms/{single_word}')
                soup = BeautifulSoup(results_synonym, "lxml")
                description_tag = soup.find("meta", property="og:description")
                if 'find any words based on your search' in description_tag['content']:
                    logger.error(f'synonym.com had no reference for the word {single_word}')
                    logger.error(f'Please verify that the word {single_word} is spelled correctly.')
                else:
                    find_synonyms = regex.split(r'\|', description_tag['content'])
                    synonyms_list = find_synonyms[2].lstrip().replace('synonyms:', '').split(',')
                    synonyms = [cleansing.normalize_space(i) for i in synonyms_list]
                    caching.insert_word_cache_synonyms(single_word, 'synonym_com', sorted(synonyms))
                    return sorted(synonyms)
            except bs4.FeatureNotFound as error:
                logger.error('An error occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except AttributeError as error:
                logger.error('An AttributeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except IndexError as error:
                logger.error('An IndexError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except KeyError as error:
                logger.error('A KeyError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except TypeError as error:
                logger.error('A TypeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            synonyms = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return synonyms
    else:
        logger.error(f'The word "{single_word}" was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')


def query_thesaurus_com(single_word):
    """
    This function queries thesaurus.com for synonyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of synonyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_synonyms(single_word, 'thesaurus_com')
        if not check_cache:
            try:
                req = requests.get(f'https://tuna.thesaurus.com/pageData/{single_word}',
                                   headers=basic_soup.http_headers,
                                   allow_redirects=True,
                                   verify=True, timeout=30)
                if '{"data":null}' not in req.text:
                    dict_synonyms = req.json()['data']['definitionData']['definitions'][0]['synonyms']
                    synonyms = sorted([r["term"] for r in dict_synonyms])
                    caching.insert_word_cache_synonyms(single_word, 'thesaurus_com', synonyms)
                    return synonyms
                else:
                    logger.error(f'The word "{single_word}" has no synonyms on thesaurus.com.')
                    logger.error(f'Please verify that the word {single_word} is spelled correctly.')
            except requests.HTTPError as error:
                logger.error('A HTTP error has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.ConnectionError as error:
                if requests.codes:
                    'Failed to establish a new connection'
                    logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.Timeout as error:
                logger.error('A connection timeout has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.RequestException as error:
                logger.error('An ambiguous exception occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            synonyms = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return synonyms
    else:
        logger.error(f'The word "{single_word}" was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')


def query_thesaurus_plus(single_word):
    """
    This function queries thesaurus.plus for synonyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of synonyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_synonyms(single_word, 'thesaurus_plus')
        if not check_cache:
            try:
                synonyms_list = []
                results_synonym = basic_soup.get_single_page_html(f'https://thesaurus.plus/synonyms/{single_word}/category/noun')
                soup = BeautifulSoup(results_synonym, "lxml")
                no_word = soup.find('title', text='404. Page not found')
                if no_word:
                    logger.error(f'thesaurus.plus has no reference for the word {single_word}')
                    logger.error(f'Please verify that the word {single_word} is spelled correctly.')
                else:
                    synonyms = []
                    parent_node = soup.find('ul', {'class': 'list paper'}).findAll('li')[1:]
                    for children in parent_node:
                        for child in children.findAll('div', {'class': 'action_pronounce'}):
                            split_dictionary = str(child.attrs).split(',')
                            synonyms_list.append(split_dictionary[1].replace("'data-term':", "").replace("'", ""))
                            synonyms = sorted([cleansing.normalize_space(i) for i in synonyms_list])
                    caching.insert_word_cache_synonyms(single_word, 'thesaurus_plus', synonyms)
                    return synonyms
            except IndexError as error:
                logger.error('A IndexError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.HTTPError as error:
                logger.error('A HTTP error has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.ConnectionError as error:
                if requests.codes:
                    'Failed to establish a new connection'
                    logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.Timeout as error:
                logger.error('A connection timeout has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.RequestException as error:
                logger.error('An ambiguous exception occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            synonyms = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return synonyms
    else:
        logger.error(f'The word "{single_word}" was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')


def query_wordnet(single_word):
    """
    This function queries wordnet for synonyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of synonyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_synonyms(single_word, 'wordnet')
        if not check_cache:
            try:
                synonyms = []
                results = requests.get(f'http://wordnetweb.princeton.edu/perl/webwn?s={single_word}',
                                       headers=basic_soup.http_headers,
                                       allow_redirects=True, verify=True, timeout=30)
                soup = BeautifulSoup(results.text, "lxml")
                if soup.findAll('h3', text='Noun'):
                    parent_node = soup.findAll("ul")[0].findAll('li')
                    for children in parent_node:
                        for child in children.find_all(href=True):
                            if 'S:' not in child.contents[0]:
                                synonyms.append(child.contents[0])
                    synonyms = sorted([x.lower() for x in synonyms])
                    caching.insert_word_cache_synonyms(single_word, 'wordnet', synonyms)
                    return synonyms
                else:
                    logger.error(f'Wordnet had no reference for the word {single_word}')
                    logger.error(f'Please verify that the word {single_word} is spelled correctly.')
            except IndexError as error:
                logger.error('A IndexError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.HTTPError as error:
                logger.error('A HTTP error has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.ConnectionError as error:
                if requests.codes:
                    'Failed to establish a new connection'
                    logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.Timeout as error:
                logger.error('A connection timeout has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.RequestException as error:
                logger.error('An ambiguous exception occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            synonyms = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return synonyms
    else:
        logger.error(f'The word "{single_word}" was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')



#!/usr/bin/env python3

"""
This Python script is designed to query internal repositories for the
homophones associated with the given word.
"""
__author__ = 'John Bumgarner'
__date__ = 'June 11, 2021'
__status__ = 'Production'
__license__ = 'MIT'
__copyright__ = "Copyright (C) 2021 John Bumgarner"

##################################################################################
# “AS-IS” Clause
#
# Except as represented in this agreement, all work produced by Developer is
# provided “AS IS”. Other than as provided in this agreement, Developer makes no
# other warranties, express or implied, and hereby disclaims all implied warranties,
# including any warranty of merchantability and warranty of fitness for a particular
# purpose.
##################################################################################

##################################################################################
# Python imports required for basic operations
##################################################################################
import os
import sys
import pickle
import logging
import traceback
import wordhoard
from wordhoard.utilities import wordhoard_logger, word_verification

logger = logging.getLogger(__name__)
wordhoard_logger.enable_logging(logger)

__PACKAGE_BASE_PATH = os.path.dirname(wordhoard.__file__)

# Opening the pickle file that contains a nested list of common
# English language homophones.
try:
    _file_known_homophones = os.path.join(__PACKAGE_BASE_PATH, 'files/common_english_homophones.pkl')
    with open(_file_known_homophones, 'rb') as _eng_homophones:
        _known_homophones_list = pickle.load(_eng_homophones)
        _eng_homophones.close()
except FileNotFoundError as error:
    logger.error('The common_english_homophones.pkl file was not found. Aborting operation.')
    logger.error(''.join(traceback.format_tb(error.__traceback__)))
    sys.exit(1)
except OSError as error:
    logger.error(f"An OS error occurred when trying to open the file common_english_homophones.pkl")
    logger.error(''.join(traceback.format_tb(error.__traceback__)))
    sys.exit(1)


# Opening the pickle file that contains a nested list of English
# language words that have no known homophones.
try:
    _file_no_known_homophones = os.path.join(__PACKAGE_BASE_PATH, 'files/no_homophones_english.pkl')
    with open(_file_no_known_homophones, 'rb') as _no_eng_homophones:
        _no_homophones_list = pickle.load(_no_eng_homophones)
        _no_eng_homophones.close()
except FileNotFoundError as error:
    logger.error('The no_homophones_english.pkl file was not found. Aborting operation.')
    logger.error(''.join(traceback.format_tb(error.__traceback__)))
    sys.exit(1)
except OSError as error:
    logger.error(f"An OS error occurred when trying to open the file no_homophones_english.pkl")
    logger.error(''.join(traceback.format_tb(error.__traceback__)))
    sys.exit(1)


def _common_english_homophones(search_string):
    """
    This function iterates through a list of known
    English language homophones.

    :param search_string: string variable to search for
    :return: list of homophones
    :rtype: list
    """
    global _known_homophones_list
    rtn_list = []
    for homophones in _known_homophones_list:
        match = bool([word for word in homophones if word == search_string])
        if match:
            for word in homophones:
                if word != search_string:
                    rtn_list.append(f'{search_string} is a homophone of {word}')
    if len(rtn_list) > 0:
        return list(set(rtn_list))


def _english_words_without_homophones(search_string):
    """
    This function iterates through a list of English
    language words with no known homophones.

    :param search_string: string variable to search for
    :return: string
    :rtype: string
    """
    global _no_homophones_list
    match = bool(search_string in _no_homophones_list)
    if match:
        return f'no homophones for {search_string}'


def query_homophones(search_string):
    """
    This function queries multiple lists to
    find an English language homophones for
    the a specific word.

    :param search_string: string variable to search for
    :return: list of homophones or string
    :rtype: list or string
    """
    valid_word = word_verification.validate_word_syntax(search_string)
    if valid_word:
        known_english_homophones = _common_english_homophones(search_string)
        if known_english_homophones:
            return known_english_homophones
        elif not known_english_homophones:
            return _english_words_without_homophones(search_string)
    else:
        logger.error(f'The word {search_string} was not in a valid format.')
        logger.error(f'Please verify that the word {search_string} is spelled correctly.')


#!/usr/bin/env python3

"""
This Python script is designed to query multiple online repositories for the
definition associated with a given word.
"""
__author__ = 'John Bumgarner'
__date__ = 'October 15, 2020'
__status__ = 'Production'
__license__ = 'MIT'
__copyright__ = "Copyright (C) 2020 John Bumgarner"

##################################################################################
# “AS-IS” Clause
#
# Except as represented in this agreement, all work produced by Developer is
# provided “AS IS”. Other than as provided in this agreement, Developer makes no
# other warranties, express or implied, and hereby disclaims all implied warranties,
# including any warranty of merchantability and warranty of fitness for a particular
# purpose.
##################################################################################

##################################################################################
# Python imports required for basic operations
##################################################################################
import bs4
import logging
import requests
import traceback
import re as regex
from bs4 import BeautifulSoup
from wordhoard.utilities import basic_soup, caching, cleansing, wordhoard_logger, word_verification

logger = logging.getLogger(__name__)
wordhoard_logger.enable_logging(logger)


def query_synonym_com(single_word):
    """
    This function queries synonym.com for a definition
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: definition for the word
    :rtype: string
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_definition(single_word, 'synonym_com')
        if not check_cache:
            try:
                results_definition = basic_soup.get_single_page_html(f'https://www.synonym.com/synonyms/{single_word}')
                soup = BeautifulSoup(results_definition, "lxml")
                description_tag = soup.find("meta", property="og:description")
                if 'find any words based on your search' not in description_tag['content']:
                    find_definition = regex.split(r'\|', description_tag['content'])
                    definition_list = find_definition[1].lstrip().replace('definition:', '').split(',')
                    definition = [cleansing.normalize_space(i) for i in definition_list]
                    definition_list_to_string = ' '.join([str(elem) for elem in definition])
                    caching.insert_word_cache_definition(single_word, 'synonym_com', definition_list_to_string)
                    return definition_list_to_string
                else:
                    logger.error(f'synonym.com had no reference for the word {single_word}')
            except bs4.FeatureNotFound as error:
                logger.error('An error occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except AttributeError as error:
                logger.error('An AttributeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except KeyError as error:
                logger.error('A KeyError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except TypeError as error:
                logger.error('A TypeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            definition = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return definition
    else:
        logger.error(f'The word "{single_word}" was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')


def query_collins_dictionary_synonym(single_word):
    """
    This function queries collinsdictionary.com for a definition
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: definition for the word
    :rtype: string
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_definition(single_word, 'collins_dictionary')
        if not check_cache:
            try:
                results_definition = basic_soup.get_single_page_html(
                    f'https://www.collinsdictionary.com/dictionary/english-thesaurus/{single_word}')
                query_results = basic_soup.query_html(results_definition, 'div', 'class',
                                                      'form type-def titleTypeSubContainer')
                if query_results is not None:
                    definition = query_results.findNext('div', {'class': 'def'})
                    caching.insert_word_cache_definition(single_word, 'collins_dictionary', definition.text)
                    return definition.text
                else:
                    logger.error(f'Collins Dictionary had no reference for the word {single_word}')
            except bs4.FeatureNotFound as error:
                logger.error('An error occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except AttributeError as error:
                logger.error('An AttributeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except KeyError as error:
                logger.error('A KeyError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except TypeError as error:
                logger.error('A TypeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            definition = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return definition
    else:
        logger.error(f'The word "{single_word}" was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')


def query_thesaurus_com(single_word):
    """
    This function queries thesaurus.com for a definition
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: definition for the word
    :rtype: string
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_definition(single_word, 'thesaurus_com')
        if not check_cache:
            try:
                req = requests.get(f'https://tuna.thesaurus.com/pageData/{single_word}',
                                   headers=basic_soup.http_headers,
                                   allow_redirects=True, verify=True, timeout=30)
                if req.json()['data'] is not None:
                    definition = req.json()['data']['definitionData']['definitions'][0]['definition']
                    caching.insert_word_cache_definition(single_word, 'thesaurus_com', definition)
                    return definition
                else:
                    logger.error(f'thesaurus.com had no reference for the word {single_word}')
            except requests.HTTPError as error:
                logger.error('A HTTP error has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.ConnectionError as error:
                if requests.codes:
                    'Failed to establish a new connection'
                    logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.Timeout as error:
                logger.error('A connection timeout has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.RequestException as error:
                logger.error('An ambiguous exception occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            definition = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return definition
    else:
        logger.error(f'The word "{single_word}" was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')




#!/usr/bin/env python3

"""
This Python script is used to enable logging within other script.
"""
__author__ = 'John Bumgarner'
__date__ = 'October 15, 2020'
__status__ = 'Production'
__license__ = 'MIT'
__copyright__ = "Copyright (C) 2020 John Bumgarner"

##################################################################################
# “AS-IS” Clause
#
# Except as represented in this agreement, all work produced by Developer is
# provided “AS IS”. Other than as provided in this agreement, Developer makes no
# other warranties, express or implied, and hereby disclaims all implied warranties,
# including any warranty of merchantability and warranty of fitness for a particular
# purpose.
##################################################################################

##################################################################################
#
# Date Completed: October 15, 2020
# Author: John Bumgarner
#
# Date Revised:
# Revised by:
#
##################################################################################

##################################################################################
# Python imports required for basic operations
##################################################################################
import logging


def enable_logging(logger):
    console = logging.StreamHandler()
    logger.addHandler(console)
    console.setLevel(logging.INFO)

    log_name = 'wordhoard_error.yaml'
    log_handler = logging.FileHandler(f'{log_name}')
    logger.addHandler(log_handler)
    formatter = logging.Formatter('%(asctime)s:%(name)s:%(levelname)s: %(message)s', '%Y-%m-%d %H:%M:%S')
    log_handler.setFormatter(formatter)
    log_handler.setLevel(logging.ERROR)










#!/usr/bin/env python3

"""
This Python script is designed to query multiple online repositories for the
antonyms associated with a specific word.
"""
__author__ = 'John Bumgarner'
__date__ = 'October 15, 2020'
__status__ = 'Production'
__license__ = 'MIT'
__copyright__ = "Copyright (C) 2020 John Bumgarner"

##################################################################################
# “AS-IS” Clause
#
# Except as represented in this agreement, all work produced by Developer is
# provided “AS IS”. Other than as provided in this agreement, Developer makes no
# other warranties, express or implied, and hereby disclaims all implied warranties,
# including any warranty of merchantability and warranty of fitness for a particular
# purpose.
##################################################################################

##################################################################################
# Python imports required for basic operations
##################################################################################
import bs4
import logging
import requests
import traceback
import re as regex
from bs4 import BeautifulSoup
from wordhoard.utilities import basic_soup, caching, cleansing, wordhoard_logger, word_verification

logger = logging.getLogger(__name__)
wordhoard_logger.enable_logging(logger)


def query_synonym_com(single_word):
    """
    This function queries synonym.com for antonyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of antonyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_antonyms(single_word, 'synonym_com')
        if not check_cache:
            try:
                results_antonyms = basic_soup.get_single_page_html(f'https://www.synonym.com/synonyms/{single_word}')
                soup = BeautifulSoup(results_antonyms, "lxml")
                description_tag = soup.find("meta", property="og:description")
                if 'find any words based on your search' in description_tag['content']:
                    logger.error(f'synonym.com had no reference for the word {single_word}')
                else:
                    find_antonyms = regex.split(r'\|', description_tag['content'])
                    antonyms_list = find_antonyms[3].lstrip().replace('antonyms:', '').split(',')
                    antonyms = sorted([cleansing.normalize_space(i) for i in antonyms_list])
                    if antonyms:
                        caching.insert_word_cache_antonyms(single_word, 'synonym_com', antonyms)
                        return antonyms
                    else:
                        logger.error(f'The word {single_word} no antonyms on synonym.com.')
            except bs4.FeatureNotFound as error:
                logger.error('An error occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except AttributeError as error:
                logger.error('An AttributeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except KeyError as error:
                logger.error('A KeyError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except TypeError as error:
                logger.error('A TypeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            antonyms = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return antonyms
    else:
        logger.error(f'The word {single_word} was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')


def query_thesaurus_com(single_word):
    """
    This function queries thesaurus.com for antonyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of antonyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_antonyms(single_word, 'thesaurus_com')
        if not check_cache:
            try:
                req = requests.get(f'https://tuna.thesaurus.com/pageData/{single_word}',
                                   headers=basic_soup.http_headers,
                                   allow_redirects=True, verify=True, timeout=30)
                if '{"data":null}' not in req.text:
                    dict_antonyms = req.json()['data']['definitionData']['definitions'][0]['antonyms']
                    if dict_antonyms:
                        antonyms = sorted([r["term"] for r in dict_antonyms])
                        caching.insert_word_cache_antonyms(single_word, 'thesaurus_com', antonyms)
                        return antonyms
                    else:
                        logger.error(f'The word {single_word} has no antonyms on thesaurus.com.')
                else:
                    logger.error(f'The word {single_word} was not found on thesaurus.com.')
            except requests.HTTPError as error:
                logger.error('A HTTP error has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.ConnectionError as error:
                if requests.codes:
                    'Failed to establish a new connection'
                    logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.Timeout as error:
                logger.error('A connection timeout has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.RequestException as error:
                logger.error('An ambiguous exception occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            antonyms = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return antonyms
    else:
        logger.error(f'The word {single_word} was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')


def query_thesaurus_plus(single_word):
    """
    This function queries thesaurus.plus for antonyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of antonyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_antonyms(single_word, 'thesaurus_plus')
        if not check_cache:
            try:
                results_antonym = basic_soup.get_single_page_html(f'https://thesaurus.plus/antonyms/{single_word}')
                soup = BeautifulSoup(results_antonym, "lxml")
                no_word = soup.find('title', text='404. Page not found')
                if no_word:
                    logger.error(f'thesaurus.plus has no reference for the word {single_word}')
                else:
                    antonyms_list = []
                    antonyms = []
                    parent_node = soup.find('ul', {'class': 'list paper'}).findAll('li')
                    for children in parent_node:
                        for child in children.findAll('div', {'class': 'action_pronounce'}):
                            split_dictionary = str(child.attrs).split(',')
                            antonyms_list.append(split_dictionary[1].replace("'data-term':", "").replace("'", ""))
                            antonyms = sorted([cleansing.normalize_space(i) for i in antonyms_list])
                    caching.insert_word_cache_antonyms(single_word, 'thesaurus_plus', antonyms)
                    return antonyms
            except IndexError as error:
                logger.error('A IndexError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
                logger.info(f'Please verify that the word {single_word} is spelled correctly.')
            except requests.HTTPError as error:
                logger.error('A HTTP error has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.ConnectionError as error:
                if requests.codes:
                    'Failed to establish a new connection'
                    logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.Timeout as error:
                logger.error('A connection timeout has occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except requests.RequestException as error:
                logger.error('An ambiguous exception occurred.')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            antonyms = cleansing.flatten_multidimensional_list([val for val in check_cache.values()])
            return antonyms
    else:
        logger.error(f'The word {single_word} was not in a valid format.')
        logger.error(f'Please verify that the word {single_word} is spelled correctly.')




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="wordhoard",
    version="1.4.1",
    author="John Bumgarner",
    author_email="wordhoardproject@gmail.com",
    description="lexical processing for textual analysis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/johnbumgarner/wordhoard",
    packages=setuptools.find_packages(),
    license='LICENSE.txt',
    classifiers=["License :: OSI Approved :: MIT License", "Operating System :: OS Independent"],
    keywords='antonyms, synonyms, definitions, lexicon, wordsearch',
    python_requires='>=3.6',
    install_requires=['bs4', 'lxml', 'requests', 'urllib3']
)



#!/usr/bin/env python3

"""
This Python script provide basic requests.get function and BeautifulSoup parent tag extraction.
"""
__author__ = 'John Bumgarner'
__date__ = 'October 15, 2020'
__status__ = 'Production'
__license__ = 'MIT'
__copyright__ = "Copyright (C) 2020 John Bumgarner"

##################################################################################
# “AS-IS” Clause
#
# Except as represented in this agreement, all work produced by Developer is
# provided “AS IS”. Other than as provided in this agreement, Developer makes no
# other warranties, express or implied, and hereby disclaims all implied warranties,
# including any warranty of merchantability and warranty of fitness for a particular
# purpose.
##################################################################################

##################################################################################
#
# Date Completed: October 15, 2020
# Author: John Bumgarner
#
# Date Revised:
# Revised by:
#
##################################################################################

##################################################################################
# Python imports required for basic operations
##################################################################################
import sys
import random
import logging
import requests
import traceback
from bs4 import BeautifulSoup
from wordhoard.utilities import wordhoard_logger

logger = logging.getLogger(__name__)
wordhoard_logger.enable_logging(logger)

##################################################################
# Array of common user agents to use for the HTTP connection
# source:  http://http://www.useragentstring.com
# source:  https://deviceatlas.com/blog/list-of-user-agent-strings
##################################################################
user_agent_strings = \
[   ##############################
    # Android Mobile User Agents #
    ##############################
    # Samsung Galaxy S9
    'Mozilla/5.0 (Linux; Android 8.0.0; SM-G960F Build/R16NW) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.84 Mobile Safari/537.36',
    # Samsung Galaxy S8
    'Mozilla/5.0 (Linux; Android 7.0; SM-G892A Build/NRD90M; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/60.0.3112.107 Mobile Safari/537.36'
    # Samsung Galaxy S8
    'Mozilla/5.0 (Linux; Android 7.0; SM-G892A Build/NRD90M; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/60.0.3112.107 Mobile Safari/537.36',
    # Samsung Galaxy S7
    'Mozilla/5.0 (Linux; Android 7.0; SM-G930VC Build/NRD90M; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/58.0.3029.83 Mobile Safari/537.36',
    # Samsung Galaxy S7 Edge
    'Mozilla/5.0 (Linux; Android 6.0.1; SM-G935S Build/MMB29K; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/55.0.2883.91 Mobile Safari/537.36',
    # Samsung Galaxy S6
    'Mozilla/5.0 (Linux; Android 6.0.1; SM-G920V Build/MMB29K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.98 Mobile Safari/537.36',
    # Samsung Galaxy S6 Edge Plus
    'Mozilla/5.0 (Linux; Android 5.1.1; SM-G928X Build/LMY47X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.83 Mobile Safari/537.36',
    # Nexus 6P
    'Mozilla/5.0 (Linux; Android 6.0.1; Nexus 6P Build/MMB29P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.83 Mobile Safari/537.36',
    # Sony Xperia XZ
    'Mozilla/5.0 (Linux; Android 7.1.1; G8231 Build/41.2.A.0.219; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/59.0.3071.125 Mobile Safari/537.36',
    # Sony Xperia Z5
    'Mozilla/5.0 (Linux; Android 6.0.1; E6653 Build/32.2.A.0.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.98 Mobile Safari/537.36',
    # HTC One X10
    'Mozilla/5.0 (Linux; Android 6.0; HTC One X10 Build/MRA58K; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/61.0.3163.98 Mobile Safari/537.36',
    # HTC One M9
    'Mozilla/5.0 (Linux; Android 6.0; HTC One M9 Build/MRA58K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.98 Mobile Safari/537.3',
    ##############################
    # iPhone User Agents         #
    ##############################
    # Apple iPhone XR (Safari)
    'Mozilla/5.0 (iPhone; CPU iPhone OS 12_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/12.0 Mobile/15E148 Safari/604.1',
    # Apple iPhone XS (Chrome)
    'Mozilla/5.0 (iPhone; CPU iPhone OS 12_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/69.0.3497.105 Mobile/15E148 Safari/605.1',
    # Apple iPhone XS Max (Firefox)
    'Mozilla/5.0 (iPhone; CPU iPhone OS 12_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) FxiOS/13.2b11866 Mobile/16A366 Safari/605.1.15',
    # Apple iPhone X
    'Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1',
    # Apple iPhone 8
    'Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.34 (KHTML, like Gecko) Version/11.0 Mobile/15A5341f Safari/604.1',
    # Apple iPhone 8 Plus
    'Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A5370a Safari/604.1',
    # Apple iPhone 7
    'Mozilla/5.0 (iPhone9,3; U; CPU iPhone OS 10_0_1 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) Version/10.0 Mobile/14A403 Safari/602.1',
    # Apple iPhone 7 Plus
    'Mozilla/5.0 (iPhone9,4; U; CPU iPhone OS 10_0_1 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) Version/10.0 Mobile/14A403 Safari/602.1',
    # Apple iPhone 6
    'Mozilla/5.0 (Apple-iPhone7C2/1202.466; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1A543 Safari/419.3',
    ################################
    # MS Windows Phone User Agents #
    ################################
    # Microsoft Lumia 650
    'Mozilla/5.0 (Windows Phone 10.0; Android 6.0.1; Microsoft; RM-1152) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.116 Mobile Safari/537.36 Edge/15.15254',
    # Microsoft Lumia 550
    'Mozilla/5.0 (Windows Phone 10.0; Android 4.2.1; Microsoft; RM-1127_16056) AppleWebKit/537.36(KHTML, like Gecko) Chrome/42.0.2311.135 Mobile Safari/537.36 Edge/12.10536',
    # Microsoft Lumia 950
    'Mozilla/5.0 (Windows Phone 10.0; Android 4.2.1; Microsoft; Lumia 950) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2486.0 Mobile Safari/537.36 Edge/13.1058',
    ######################
    # Tablet User Agents #
    ######################
    # Google Pixel C
    'Mozilla/5.0 (Linux; Android 7.0; Pixel C Build/NRD90M; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/52.0.2743.98 Safari/537.36',
    # Sony Xperia Z4 Tablet
    'Mozilla/5.0 (Linux; Android 6.0.1; SGP771 Build/32.2.A.0.253; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/52.0.2743.98 Safari/537.36',
    # Nvidia Shield Tablet K1
    'Mozilla/5.0 (Linux; Android 6.0.1; SHIELD Tablet K1 Build/MRA58K; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/55.0.2883.91 Safari/537.36',
    # Samsung Galaxy Tab S3
    'Mozilla/5.0 (Linux; Android 7.0; SM-T827R4 Build/NRD90M) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.116 Safari/537.36',
    # Samsung Galaxy Tab A
    'Mozilla/5.0 (Linux; Android 5.0.2; SAMSUNG SM-T550 Build/LRX22G) AppleWebKit/537.36 (KHTML, like Gecko) SamsungBrowser/3.3 Chrome/38.0.2125.102 Safari/537.36',
    # Amazon Kindle Fire HDX 7
    'Mozilla/5.0 (Linux; Android 4.4.3; KFTHWI Build/KTU84M) AppleWebKit/537.36 (KHTML, like Gecko) Silk/47.1.79 like Chrome/47.0.2526.80 Safari/537.36',
    # LG G Pad 7.0
    'Mozilla/5.0 (Linux; Android 5.0.2; LG-V410/V41020c Build/LRX22G) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/34.0.1847.118 Safari/537.36',
    #######################
    # Desktop User Agents #
    #######################
    # Windows 10-based PC using Edge browser
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.135 Safari/537.36 Edge/12.246',
    # Chrome OS-based laptop using Chrome browser (Chromebook)
    'Mozilla/5.0 (X11; CrOS x86_64 8172.45.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.64 Safari/537.36',
    # Mac OS X-based computer using a Safari browser
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/601.3.9 (KHTML, like Gecko) Version/9.0.2 Safari/601.3.9',
    # Windows 7-based PC using a Chrome browser
    'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.111 Safari/537.36',
    # Linux-based PC using a Firefox browser
    'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0) Gecko/20100101 Firefox/15.0.1',
    # Chrome 70.0.3538.77
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36',
    # Chrome 75.0.3770.90
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.90 Safari/537.36',
    # Firefox 64.0
    'Mozilla/5.0 (X11; Linux i686; rv:64.0) Gecko/20100101 Firefox/64.0',
    'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:64.0) Gecko/20100101 Firefox/64.0',
    # Firefox 63.0
    'Mozilla/5.0 (X11; Linux i586; rv:63.0) Gecko/20100101 Firefox/63.0',
    'Mozilla/5.0 (Windows NT 6.2; WOW64; rv:63.0) Gecko/20100101 Firefox/63.0',
    # Firefox 62.0
    'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.10; rv:62.0) Gecko/20100101 Firefox/62.0',
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:10.0) Gecko/20100101 Firefox/62.0',
    # Internet Explorer 11.0
    'Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; AS; rv:11.0) like Gecko',
    'Mozilla/5.0 (compatible, MSIE 11, Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko'
]

##################################################################
# Select a random user agent from the array of user agent choices
##################################################################
rand_user_agent = random.choice(user_agent_strings)

############################################
# Create http request for harvest operation
############################################
http_headers = {'user-agent': rand_user_agent}


###################################################################
# Open a HTTP connection and harvest HTML from initial source URL
###################################################################
def get_single_page_html(url_to_scrape):
    with requests.Session() as session:
        try:
            raw_html = session.get(url_to_scrape, headers=http_headers, allow_redirects=True, verify=True, timeout=30)
        except requests.HTTPError as e:
            logger.error(f'A HTTPError has occurred when requesting {url_to_scrape}')
            logger.error(''.join(traceback.format_tb(e.__traceback__)))
            sys.exit(1)
        except requests.URLRequired as e:
            logger.error(f'A URLRequired has occurred when requesting {url_to_scrape}')
            logger.error(''.join(traceback.format_tb(e.__traceback__)))
            sys.exit(1)
        except requests.ConnectionError as e:
            if requests.codes: 'Failed to establish a new connection'
            logger.error(f'A ConnectionError has occurred when requesting {url_to_scrape}')
            logger.error(''.join(traceback.format_tb(e.__traceback__)))
            sys.exit(1)
        except requests.Timeout as e:
            logger.error(f'A Timeout has occurred when requesting {url_to_scrape}')
            logger.error(''.join(traceback.format_tb(e.__traceback__)))
            sys.exit(1)
        except requests.RequestException as e:
            logger.error(f'A RequestException has occurred when requesting {url_to_scrape}')
            logger.error(''.join(traceback.format_tb(e.__traceback__)))
            sys.exit(1)
        return raw_html.text


def query_html(raw_html, tag_type, tag_attribute, attribute_text):
    """
    :param raw_html: The HTML code for the paged that was scraped in the previous function
    :param tag_type:
    :param tag_attribute:
    :param attribute_text:
    :return: The section of the HTML containing the information that requires further parsing
    """
    soup = BeautifulSoup(raw_html, 'lxml')
    try:
        results = soup.find(tag_type, {tag_attribute: attribute_text})
    except AttributeError as e:
        logger.error('An AttributeError has occurred when parsing with BeautifulSoup.')
        logger.error(''.join(traceback.format_tb(e.__traceback__)))
        sys.exit(1)
    return results



#!/usr/bin/env python3

"""
This Python script is to create temporary dictionary caches.  These caches are
designed to limit redundant queries.
"""
__author__ = 'John Bumgarner'
__date__ = 'October 15, 2020'
__status__ = 'Production'
__license__ = 'MIT'
__copyright__ = "Copyright (C) 2020 John Bumgarner"

##################################################################################
# “AS-IS” Clause
#
# Except as represented in this agreement, all work produced by Developer is
# provided “AS IS”. Other than as provided in this agreement, Developer makes no
# other warranties, express or implied, and hereby disclaims all implied warranties,
# including any warranty of merchantability and warranty of fitness for a particular
# purpose.
##################################################################################

##################################################################################
#
# Date Completed: October 15, 2020
# Author: John Bumgarner
#
# Date Revised: June 1, 2021
# Revised by: John Bumgarner
#
##################################################################################

##################################################################################
# in memory temporary cache for antonyms
##################################################################################
temporary_dict_antonyms = {}


def cache_antonyms(word, source):
    item_to_check = f'{word}_{source}'
    if item_to_check in temporary_dict_antonyms.keys():
        new_dict = {key: val for (key, val) in temporary_dict_antonyms.items() if key == word}
        return new_dict
    else:
        return False


def insert_word_cache_antonyms(word, source, values):
    key = f'{word}_{source}'
    temporary_dict_antonyms[key] = values


##################################################################################
# in memory temporary cache for synonyms
##################################################################################
temporary_dict_synonyms = {}


def cache_synonyms(word, source):
    item_to_check = f'{word}_{source}'
    if item_to_check in temporary_dict_synonyms.keys():
        new_dict = {key: val for (key, val) in temporary_dict_synonyms.items() if key == word}
        return new_dict
    else:
        return False


def insert_word_cache_synonyms(word, source, values):
    key = f'{word}_{source}'
    temporary_dict_synonyms[key] = values


##################################################################################
# in memory temporary cache for definitions
##################################################################################
temporary_dict_definition = {}


def cache_definition(word, source):
    item_to_check = f'{word}_{source}'
    if item_to_check in temporary_dict_definition.keys():
        new_dict = {key: val for (key, val) in temporary_dict_synonyms.items() if key == word}
        return new_dict
    else:
        return False


def insert_word_cache_definition(word, source, values):
    key = f'{word}_{source}'
    temporary_dict_definition[key] = values


##################################################################################
# in memory temporary cache for hypernyms
##################################################################################
temporary_dict_hypernyms = {}


def cache_hypernyms(word, source):
    item_to_check = f'{word}_{source}'
    if item_to_check in temporary_dict_hypernyms.keys():
        new_dict = {key: val for (key, val) in temporary_dict_hypernyms.items() if key == word}
        return new_dict
    else:
        return False


def insert_word_cache_hypernyms(word, source, values):
    key = f'{word}_{source}'
    temporary_dict_hypernyms[key] = values


##################################################################################
# in memory temporary cache for hyponyms
##################################################################################
temporary_dict_hyponyms = {}


def cache_hyponyms(word, source):
    item_to_check = f'{word}_{source}'
    if item_to_check in temporary_dict_hyponyms.keys():
        new_dict = {key: val for (key, val) in temporary_dict_hyponyms.items() if key == word}
        return new_dict
    else:
        return False


def insert_word_cache_hyponyms(word, source, values):
    key = f'{word}_{source}'
    temporary_dict_hyponyms[key] = values




#!/usr/bin/env python3

"""
This Python script is designed to query a online repository for the
hyponyms associated with a specific word.
"""
__author__ = 'John Bumgarner'
__date__ = 'June 11, 2021'
__status__ = 'Production'
__license__ = 'MIT'
__copyright__ = "Copyright (C) 2021 John Bumgarner"

##################################################################################
# “AS-IS” Clause
#
# Except as represented in this agreement, all work produced by Developer is
# provided “AS IS”. Other than as provided in this agreement, Developer makes no
# other warranties, express or implied, and hereby disclaims all implied warranties,
# including any warranty of merchantability and warranty of fitness for a particular
# purpose.
##################################################################################

##################################################################################
# Python imports required for basic operations
##################################################################################
import bs4
import logging
import requests
import traceback
from bs4 import BeautifulSoup
from wordhoard.utilities import basic_soup, caching, wordhoard_logger, word_verification

logger = logging.getLogger(__name__)
wordhoard_logger.enable_logging(logger)


def _get_number_of_pages(soup):
    """
    This function determines the number of pages
    that contain hyponyms for a specific word.

    :param soup: BeautifulSoup lxml
    :return: number of pages
    :rtype: int
    """
    try:
        number_of_pages = 0
        pages = soup.find('div', {'id': 'pages'})
        if pages is not None:
            list_of_pages = [num for page in pages for num in page if num.isdigit()]
            if len(list_of_pages) != 0:
                number_of_pages = int(list_of_pages[-1]) + 1
        return number_of_pages

    except AttributeError as error:
        logger.error('An AttributeError occurred in the following code segment:')
        logger.error(''.join(traceback.format_tb(error.__traceback__)))
    except KeyError as error:
        logger.error('A KeyError occurred in the following code segment:')
        logger.error(''.join(traceback.format_tb(error.__traceback__)))
    except TypeError as error:
        logger.error('A TypeError occurred in the following code segment:')
        logger.error(''.join(traceback.format_tb(error.__traceback__)))


def _get_hyponyms(soup):
    """
    This function queries a table for hyponyms.

    :param soup: BeautifulSoup lxml
    :return: set of hyponyms
    :rtype: set
    """
    try:
        sub_set = set()
        table = soup.find('table')
        if table:
            rows = table.find_all('tr', {'class': 'theentry'})
            if rows is not None:
                for row in rows:
                    cols = row.find('td', {'class': 'abbdef'}).find('a')
                    if cols is not None:
                        if cols.text != '»':
                            sub_set.add(str(cols.text).lower())
                        else:
                            sub_set.add('no hyponyms found')
        return sub_set

    except AttributeError as error:
        logger.error('An AttributeError occurred in the following code segment:')
        logger.error(''.join(traceback.format_tb(error.__traceback__)))
    except KeyError as error:
        logger.error('A KeyError occurred in the following code segment:')
        logger.error(''.join(traceback.format_tb(error.__traceback__)))
    except TypeError as error:
        logger.error('A TypeError occurred in the following code segment:')
        logger.error(''.join(traceback.format_tb(error.__traceback__)))


def find_hyponyms(single_word):
    """
    This function queries classicthesaurus_com for hyponyms
    related to the 'single_word' parameter.

    :param single_word: string variable to search for
    :return: list of hyponyms
    :rtype: list
    """
    valid_word = word_verification.validate_word_syntax(single_word)
    if valid_word:
        check_cache = caching.cache_hyponyms(single_word, 'classicthesaurus_com')
        if not check_cache:
            try:
                results_hyponyms = basic_soup.get_single_page_html(f'https://www.classicthesaurus.com/{single_word}/narrower')
                soup = BeautifulSoup(results_hyponyms, "lxml")
                hyponym = _get_hyponyms(soup)

                number_of_pages = _get_number_of_pages(soup)
                if number_of_pages >= 2:
                    for page in range(2, number_of_pages):
                        sub_html = requests.get(f'https://www.classicthesaurus.com/{single_word}/narrower/{page}',
                                                headers=basic_soup.http_headers)
                        sub_soup = BeautifulSoup(sub_html.text, 'lxml')
                        additional_hyponym = _get_hyponyms(sub_soup)
                        hyponym.union(additional_hyponym)

                return sorted(hyponym)

            except bs4.FeatureNotFound as error:
                logger.error('An error occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except AttributeError as error:
                logger.error('An AttributeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except KeyError as error:
                logger.error('A KeyError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
            except TypeError as error:
                logger.error('A TypeError occurred in the following code segment:')
                logger.error(''.join(traceback.format_tb(error.__traceback__)))
        else:
            logger.error(f'The word {single_word} was not in a valid format.')
            logger.error(f'Please verify that the word {single_word} is spelled correctly.')

